@@ -1,4 +1,3 @@
-{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE PolyKinds #-}
@@ -9,10 +8,14 @@
 module Witch.Instances where
 
 import qualified Control.Exception as Exception
+import qualified Control.Monad as Monad
 import qualified Data.Bits as Bits
 import qualified Data.ByteString as ByteString
+import qualified Data.ByteString.Char8 as Char8
 import qualified Data.ByteString.Lazy as LazyByteString
+import qualified Data.ByteString.Lazy.Char8 as LazyChar8
 import qualified Data.ByteString.Short as ShortByteString
+import qualified Data.Char as Char
 import qualified Data.Complex as Complex
 import qualified Data.Fixed as Fixed
 import qualified Data.Foldable as Foldable
@@ -28,6 +31,7 @@ import qualified Data.Set as Set
 import qualified Data.Tagged as Tagged
 import qualified Data.Text as Text
 import qualified Data.Text.Encoding as Text
+import qualified Data.Text.Encoding.Error as Text
 import qualified Data.Text.Lazy as LazyText
 import qualified Data.Text.Lazy.Encoding as LazyText
 import qualified Data.Time as Time
@@ -38,6 +42,8 @@ import qualified Data.Word as Word
 import qualified GHC.Float as Float
 import qualified Numeric
 import qualified Numeric.Natural as Natural
+import qualified System.IO.Unsafe as Unsafe
+import qualified Witch.Encoding as Encoding
 import qualified Witch.From as From
 import qualified Witch.TryFrom as TryFrom
 import qualified Witch.TryFromException as TryFromException
@@ -905,7 +911,7 @@ instance From.From Double Float where
 -- Ratio
 
 -- | Uses '(Ratio.%)' with a denominator of 1.
-instance Integral a => From.From a (Ratio.Ratio a) where
+instance (Integral a) => From.From a (Ratio.Ratio a) where
   from = (Ratio.% 1)
 
 -- | Uses 'Ratio.numerator' when the denominator is 1.
@@ -924,7 +930,7 @@ instance From.From Rational Double where
   from = fromRational
 
 -- | Uses `fromRational` as long as there isn't a loss of precision.
-instance Fixed.HasResolution a => TryFrom.TryFrom Rational (Fixed.Fixed a) where
+instance (Fixed.HasResolution a) => TryFrom.TryFrom Rational (Fixed.Fixed a) where
   tryFrom = Utility.eitherTryFrom $ \s ->
     let t :: Fixed.Fixed a
         t = fromRational s
@@ -943,13 +949,13 @@ instance From.From (Fixed.Fixed a) Integer where
   from (Fixed.MkFixed t) = t
 
 -- | Uses 'toRational'.
-instance Fixed.HasResolution a => From.From (Fixed.Fixed a) Rational where
+instance (Fixed.HasResolution a) => From.From (Fixed.Fixed a) Rational where
   from = toRational
 
 -- Complex
 
 -- | Uses '(Complex.:+)' with an imaginary part of 0.
-instance Num a => From.From a (Complex.Complex a) where
+instance (Num a) => From.From a (Complex.Complex a) where
   from = (Complex.:+ 0)
 
 -- | Uses 'Complex.realPart' when the imaginary part is 0.
@@ -972,7 +978,7 @@ instance From.From (NonEmpty.NonEmpty a) [a] where
 -- Set
 
 -- | Uses 'Set.fromList'.
-instance Ord a => From.From [a] (Set.Set a) where
+instance (Ord a) => From.From [a] (Set.Set a) where
   from = Set.fromList
 
 -- | Uses 'Set.toAscList'.
@@ -993,7 +999,7 @@ instance From.From IntSet.IntSet [Int] where
 
 -- | Uses 'Map.fromList'. If there are duplicate keys, later values will
 -- overwrite earlier ones.
-instance Ord k => From.From [(k, v)] (Map.Map k v) where
+instance (Ord k) => From.From [(k, v)] (Map.Map k v) where
   from = Map.fromList
 
 -- | Uses 'Map.toAscList'.
@@ -1039,42 +1045,6 @@ instance From.From ByteString.ByteString LazyByteString.ByteString where
 instance From.From ByteString.ByteString ShortByteString.ShortByteString where
   from = ShortByteString.toShort
 
--- | Uses 'Text.decodeUtf8''.
-instance TryFrom.TryFrom ByteString.ByteString Text.Text where
-  tryFrom = Utility.eitherTryFrom Text.decodeUtf8'
-
--- | Uses 'Text.decodeUtf8''.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) Text.Text where
-  tryFrom = Utility.eitherTryFrom $ Text.decodeUtf8' . From.from
-
--- | Converts via 'Text.Text'.
-instance TryFrom.TryFrom ByteString.ByteString LazyText.Text where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @LazyText.Text)
-        . Utility.tryInto @Text.Text
-
--- | Converts via 'Text.Text'.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) LazyText.Text where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @LazyText.Text)
-        . Utility.tryInto @Text.Text
-
--- | Converts via 'Text.Text'.
-instance TryFrom.TryFrom ByteString.ByteString String where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @String)
-        . Utility.tryInto @Text.Text
-
--- | Converts via 'Text.Text'.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) String where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @String)
-        . Utility.tryInto @Text.Text
-
 -- LazyByteString
 
 -- | Uses 'LazyByteString.pack'.
@@ -1089,42 +1059,6 @@ instance From.From LazyByteString.ByteString [Word.Word8] where
 instance From.From LazyByteString.ByteString ByteString.ByteString where
   from = LazyByteString.toStrict
 
--- | Uses 'LazyText.decodeUtf8''.
-instance TryFrom.TryFrom LazyByteString.ByteString LazyText.Text where
-  tryFrom = Utility.eitherTryFrom LazyText.decodeUtf8'
-
--- | Uses 'LazyText.decodeUtf8''.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) LazyText.Text where
-  tryFrom = Utility.eitherTryFrom $ LazyText.decodeUtf8' . From.from
-
--- | Converts via 'LazyText.Text'.
-instance TryFrom.TryFrom LazyByteString.ByteString Text.Text where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @Text.Text)
-        . Utility.tryInto @LazyText.Text
-
--- | Converts via 'LazyText.Text'.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) Text.Text where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @Text.Text)
-        . Utility.tryInto @LazyText.Text
-
--- | Converts via 'LazyText.Text'.
-instance TryFrom.TryFrom LazyByteString.ByteString String where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @String)
-        . Utility.tryInto @LazyText.Text
-
--- | Converts via 'LazyText.Text'.
-instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) String where
-  tryFrom =
-    Utility.eitherTryFrom $
-      fmap (Utility.into @String)
-        . Utility.tryInto @LazyText.Text
-
 -- ShortByteString
 
 -- | Uses 'ShortByteString.pack'.
@@ -1145,44 +1079,12 @@ instance From.From ShortByteString.ShortByteString ByteString.ByteString where
 instance From.From Text.Text LazyText.Text where
   from = LazyText.fromStrict
 
--- | Uses 'Text.encodeUtf8'.
-instance From.From Text.Text ByteString.ByteString where
-  from = Text.encodeUtf8
-
--- | Uses 'Text.encodeUtf8'.
-instance From.From Text.Text (Tagged.Tagged "UTF-8" ByteString.ByteString) where
-  from = From.from . Text.encodeUtf8
-
--- | Converts via 'ByteString.ByteString'.
-instance From.From Text.Text LazyByteString.ByteString where
-  from = Utility.via @ByteString.ByteString
-
--- | Converts via 'ByteString.ByteString'.
-instance From.From Text.Text (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
-  from = fmap From.from . Utility.into @(Tagged.Tagged "UTF-8" ByteString.ByteString)
-
 -- LazyText
 
 -- | Uses 'LazyText.toStrict'.
 instance From.From LazyText.Text Text.Text where
   from = LazyText.toStrict
 
--- | Uses 'LazyText.encodeUtf8'.
-instance From.From LazyText.Text LazyByteString.ByteString where
-  from = LazyText.encodeUtf8
-
--- | Uses 'LazyText.encodeUtf8'.
-instance From.From LazyText.Text (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
-  from = From.from . LazyText.encodeUtf8
-
--- | Converts via 'LazyByteString.ByteString'.
-instance From.From LazyText.Text ByteString.ByteString where
-  from = Utility.via @LazyByteString.ByteString
-
--- | Converts via 'LazyByteString.ByteString'.
-instance From.From LazyText.Text (Tagged.Tagged "UTF-8" ByteString.ByteString) where
-  from = fmap From.from . Utility.into @(Tagged.Tagged "UTF-8" LazyByteString.ByteString)
-
 -- String
 
 -- | Uses 'Text.pack'. Some 'Char' values cannot be represented in 'Text.Text'
@@ -1203,22 +1105,6 @@ instance From.From String LazyText.Text where
 instance From.From LazyText.Text String where
   from = LazyText.unpack
 
--- | Converts via 'Text.Text'.
-instance From.From String ByteString.ByteString where
-  from = Utility.via @Text.Text
-
--- | Converts via 'Text.Text'.
-instance From.From String (Tagged.Tagged "UTF-8" ByteString.ByteString) where
-  from = Utility.via @Text.Text
-
--- | Converts via 'LazyText.Text'.
-instance From.From String LazyByteString.ByteString where
-  from = Utility.via @LazyText.Text
-
--- | Converts via 'LazyText.Text'.
-instance From.From String (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
-  from = Utility.via @LazyText.Text
-
 -- TryFromException
 
 -- | Uses @coerce@.
@@ -1346,10 +1232,314 @@ instance From.From (Tagged.Tagged t a) a
 -- | Uses @coerce@. Essentially the same as 'Tagged.retag'.
 instance From.From (Tagged.Tagged t a) (Tagged.Tagged u a)
 
+-- ISO-8859-1
+
+-- | Uses 'Text.decodeLatin1'.
+instance From.From (Encoding.ISO_8859_1 ByteString.ByteString) Text.Text where
+  from = Text.decodeLatin1 . From.from
+
+-- | Converts via 'Text.Text'.
+instance From.From (Encoding.ISO_8859_1 ByteString.ByteString) LazyText.Text where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance From.From (Encoding.ISO_8859_1 ByteString.ByteString) String where
+  from = Utility.via @Text.Text
+
+-- | Uses 'LazyText.decodeLatin1'.
+instance From.From (Encoding.ISO_8859_1 LazyByteString.ByteString) LazyText.Text where
+  from = LazyText.decodeLatin1 . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance From.From (Encoding.ISO_8859_1 LazyByteString.ByteString) Text.Text where
+  from = Utility.via @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From (Encoding.ISO_8859_1 LazyByteString.ByteString) String where
+  from = Utility.via @LazyText.Text
+
+-- | Converts via 'String'.
+instance TryFrom.TryFrom Text.Text (Encoding.ISO_8859_1 ByteString.ByteString) where
+  tryFrom = Utility.eitherTryFrom $ TryFrom.tryFrom . Utility.into @String
+
+-- | Converts via 'String'.
+instance TryFrom.TryFrom Text.Text (Encoding.ISO_8859_1 LazyByteString.ByteString) where
+  tryFrom = Utility.eitherTryFrom $ TryFrom.tryFrom . Utility.into @String
+
+-- | Converts via 'String'.
+instance TryFrom.TryFrom LazyText.Text (Encoding.ISO_8859_1 LazyByteString.ByteString) where
+  tryFrom = Utility.eitherTryFrom $ TryFrom.tryFrom . Utility.into @String
+
+-- | Converts via 'String'.
+instance TryFrom.TryFrom LazyText.Text (Encoding.ISO_8859_1 ByteString.ByteString) where
+  tryFrom = Utility.eitherTryFrom $ TryFrom.tryFrom . Utility.into @String
+
+-- | Uses 'Char8.pack' when each character 'Char.isLatin1'.
+instance TryFrom.TryFrom String (Encoding.ISO_8859_1 ByteString.ByteString) where
+  tryFrom = Utility.maybeTryFrom $ \string -> do
+    Monad.guard $ all Char.isLatin1 string
+    pure . From.from $ Char8.pack string
+
+-- | Uses 'LazyChar8.pack' when each character 'Char.isLatin1'.
+instance TryFrom.TryFrom String (Encoding.ISO_8859_1 LazyByteString.ByteString) where
+  tryFrom = Utility.maybeTryFrom $ \string -> do
+    Monad.guard $ all Char.isLatin1 string
+    pure . From.from $ LazyChar8.pack string
+
+-- UTF-8
+
+-- | Uses 'Text.decodeUtf8''.
+instance TryFrom.TryFrom (Encoding.UTF_8 ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ Text.decodeUtf8' . From.from
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_8 ByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @LazyText.Text) . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_8 ByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @Text.Text
+
+-- | Uses 'LazyText.decodeUtf8''.
+instance TryFrom.TryFrom (Encoding.UTF_8 LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ LazyText.decodeUtf8' . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_8 LazyByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @Text.Text) . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_8 LazyByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @LazyText.Text
+
+-- | Uses 'Text.encodeUtf8'.
+instance From.From Text.Text (Encoding.UTF_8 ByteString.ByteString) where
+  from = From.from . Text.encodeUtf8
+
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Encoding.UTF_8 LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_8 ByteString.ByteString)
+
+-- | Uses 'LazyText.encodeUtf8'.
+instance From.From LazyText.Text (Encoding.UTF_8 LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf8
+
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Encoding.UTF_8 ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_8 LazyByteString.ByteString)
+
+-- | Converts via 'Text.Text'.
+instance From.From String (Encoding.UTF_8 ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Encoding.UTF_8 LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
+-- UTF-16LE
+
+-- | Uses 'Text.decodeUtf16LE'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . Text.decodeUtf16LE . From.from
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE ByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @LazyText.Text) . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE ByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @Text.Text
+
+-- | Uses 'LazyText.decodeUtf16LE'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . LazyText.decodeUtf16LE . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE LazyByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @Text.Text) . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16LE LazyByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @LazyText.Text
+
+-- | Uses 'Text.encodeUtf16LE'.
+instance From.From Text.Text (Encoding.UTF_16LE ByteString.ByteString) where
+  from = From.from . Text.encodeUtf16LE
+
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Encoding.UTF_16LE LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_16LE ByteString.ByteString)
+
+-- | Uses 'LazyText.encodeUtf16LE'.
+instance From.From LazyText.Text (Encoding.UTF_16LE LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf16LE
+
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Encoding.UTF_16LE ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_16LE LazyByteString.ByteString)
+
+-- | Converts via 'Text.Text'.
+instance From.From String (Encoding.UTF_16LE ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Encoding.UTF_16LE LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
+-- UTF-16BE
+
+-- | Uses 'Text.decodeUtf16BE'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . Text.decodeUtf16BE . From.from
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE ByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @LazyText.Text) . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE ByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @Text.Text
+
+-- | Uses 'LazyText.decodeUtf16BE'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . LazyText.decodeUtf16BE . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE LazyByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @Text.Text) . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_16BE LazyByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @LazyText.Text
+
+-- | Uses 'Text.encodeUtf16BE'.
+instance From.From Text.Text (Encoding.UTF_16BE ByteString.ByteString) where
+  from = From.from . Text.encodeUtf16BE
+
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Encoding.UTF_16BE LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_16BE ByteString.ByteString)
+
+-- | Uses 'LazyText.encodeUtf16BE'.
+instance From.From LazyText.Text (Encoding.UTF_16BE LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf16BE
+
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Encoding.UTF_16BE ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_16BE LazyByteString.ByteString)
+
+-- | Converts via 'Text.Text'.
+instance From.From String (Encoding.UTF_16BE ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Encoding.UTF_16BE LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
+-- UTF-32LE
+
+-- | Uses 'Text.decodeUtf32LE'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . Text.decodeUtf32LE . From.from
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE ByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @LazyText.Text) . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE ByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @Text.Text
+
+-- | Uses 'LazyText.decodeUtf32LE'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . LazyText.decodeUtf32LE . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE LazyByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @Text.Text) . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32LE LazyByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @LazyText.Text
+
+-- | Uses 'Text.encodeUtf32LE'.
+instance From.From Text.Text (Encoding.UTF_32LE ByteString.ByteString) where
+  from = From.from . Text.encodeUtf32LE
+
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Encoding.UTF_32LE LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_32LE ByteString.ByteString)
+
+-- | Uses 'LazyText.encodeUtf32LE'.
+instance From.From LazyText.Text (Encoding.UTF_32LE LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf32LE
+
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Encoding.UTF_32LE ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_32LE LazyByteString.ByteString)
+
+-- | Converts via 'Text.Text'.
+instance From.From String (Encoding.UTF_32LE ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Encoding.UTF_32LE LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
+-- UTF-32BE
+
+-- | Uses 'Text.decodeUtf32BE'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . Text.decodeUtf32BE . From.from
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE ByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @LazyText.Text) . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE ByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @Text.Text
+
+-- | Uses 'LazyText.decodeUtf32BE'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ tryEvaluate @Text.UnicodeException . LazyText.decodeUtf32BE . From.from
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE LazyByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @Text.Text) . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Encoding.UTF_32BE LazyByteString.ByteString) String where
+  tryFrom = Utility.eitherTryFrom $ fmap (Utility.into @String) . Utility.tryInto @LazyText.Text
+
+-- | Uses 'Text.encodeUtf32BE'.
+instance From.From Text.Text (Encoding.UTF_32BE ByteString.ByteString) where
+  from = From.from . Text.encodeUtf32BE
+
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Encoding.UTF_32BE LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_32BE ByteString.ByteString)
+
+-- | Uses 'LazyText.encodeUtf32BE'.
+instance From.From LazyText.Text (Encoding.UTF_32BE LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf32BE
+
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Encoding.UTF_32BE ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Encoding.UTF_32BE LazyByteString.ByteString)
+
+-- | Converts via 'Text.Text'.
+instance From.From String (Encoding.UTF_32BE ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Encoding.UTF_32BE LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
 --
 
 realFloatToRational ::
-  RealFloat s => s -> Either Exception.ArithException Rational
+  (RealFloat s) => s -> Either Exception.ArithException Rational
 realFloatToRational s
   | isNaN s = Left Exception.LossOfPrecision
   | isInfinite s =
@@ -1379,10 +1569,13 @@ fromNonNegativeIntegral x =
 
 -- | The maximum integral value that can be unambiguously represented as a
 -- 'Float'. Equal to 16,777,215.
-maxFloat :: Num a => a
+maxFloat :: (Num a) => a
 maxFloat = 16777215
 
 -- | The maximum integral value that can be unambiguously represented as a
 -- 'Double'. Equal to 9,007,199,254,740,991.
-maxDouble :: Num a => a
+maxDouble :: (Num a) => a
 maxDouble = 9007199254740991
+
+tryEvaluate :: (Exception.Exception e) => a -> Either e a
+tryEvaluate = Unsafe.unsafePerformIO . Exception.try . Exception.evaluate
