@@ -33,6 +33,8 @@ import qualified GHC.Stack as Stack
 import qualified Numeric.Natural as Natural
 import qualified Test.HUnit as HUnit
 import qualified Witch
+import qualified Witch.Encoding as Encoding
+import qualified Witch.Utility as Utility
 
 main :: IO ()
 main = HUnit.runTestTTAndExit $ specToTest spec
@@ -54,7 +56,7 @@ spec = describe "Witch" $ do
   describe "Utility" $ do
     describe "as" $ do
       it "works" $ do
-        Witch.as @Int.Int8 1 `shouldBe` 1
+        Utility.as @Int.Int8 1 `shouldBe` 1
 
     describe "into" $ do
       it "works" $ do
@@ -62,7 +64,7 @@ spec = describe "Witch" $ do
 
     describe "over" $ do
       it "works" $ do
-        Witch.over @Int.Int8 (+ 1) (Age 1) `shouldBe` Age 2
+        Utility.over @Int.Int8 (+ 1) (Age 1) `shouldBe` Age 2
 
     describe "via" $ do
       it "works" $ do
@@ -1800,48 +1802,6 @@ spec = describe "Witch" $ do
         f (ByteString.pack [0x00]) `shouldBe` ShortByteString.pack [0x00]
         f (ByteString.pack [0x0f, 0xf0]) `shouldBe` ShortByteString.pack [0x0f, 0xf0]
 
-    describe "TryFrom ByteString Text" $ do
-      let f = hush . Witch.tryFrom @ByteString.ByteString @Text.Text
-      it "works" $ do
-        f (ByteString.pack []) `shouldBe` Just (Text.pack "")
-        f (ByteString.pack [0x61]) `shouldBe` Just (Text.pack "a")
-        f (ByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom ByteString Text" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" ByteString.ByteString) @Text.Text
-      it "works" $ do
-        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
-        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just (Text.pack "a")
-        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
-
-    describe "TryFrom ByteString LazyText" $ do
-      let f = hush . Witch.tryFrom @ByteString.ByteString @LazyText.Text
-      it "works" $ do
-        f (ByteString.pack []) `shouldBe` Just (LazyText.pack "")
-        f (ByteString.pack [0x61]) `shouldBe` Just (LazyText.pack "a")
-        f (ByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom ByteString LazyText" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" ByteString.ByteString) @LazyText.Text
-      it "works" $ do
-        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (LazyText.pack "")
-        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just (LazyText.pack "a")
-        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
-
-    describe "TryFrom ByteString String" $ do
-      let f = hush . Witch.tryFrom @ByteString.ByteString @String
-      it "works" $ do
-        f (ByteString.pack []) `shouldBe` Just ""
-        f (ByteString.pack [0x61]) `shouldBe` Just "a"
-        f (ByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom ByteString String" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" ByteString.ByteString) @String
-      it "works" $ do
-        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just ""
-        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just "a"
-        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
-
     describe "From [Word8] LazyByteString" $ do
       let f = Witch.from @[Word.Word8] @LazyByteString.ByteString
       it "works" $ do
@@ -1863,48 +1823,6 @@ spec = describe "Witch" $ do
         f (LazyByteString.pack [0x00]) `shouldBe` ByteString.pack [0x00]
         f (LazyByteString.pack [0x0f, 0xf0]) `shouldBe` ByteString.pack [0x0f, 0xf0]
 
-    describe "TryFrom LazyByteString LazyText" $ do
-      let f = hush . Witch.tryFrom @LazyByteString.ByteString @LazyText.Text
-      it "works" $ do
-        f (LazyByteString.pack []) `shouldBe` Just (LazyText.pack "")
-        f (LazyByteString.pack [0x61]) `shouldBe` Just (LazyText.pack "a")
-        f (LazyByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom LazyByteString Text" $ do
-      let f = hush . Witch.tryFrom @LazyByteString.ByteString @Text.Text
-      it "works" $ do
-        f (LazyByteString.pack []) `shouldBe` Just (Text.pack "")
-        f (LazyByteString.pack [0x61]) `shouldBe` Just (Text.pack "a")
-        f (LazyByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom LazyByteString LazyText" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" LazyByteString.ByteString) @LazyText.Text
-      it "works" $ do
-        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just (LazyText.pack "")
-        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just (LazyText.pack "a")
-        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
-
-    describe "TryFrom LazyByteString Text" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" LazyByteString.ByteString) @Text.Text
-      it "works" $ do
-        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just (Text.pack "")
-        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just (Text.pack "a")
-        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
-
-    describe "TryFrom LazyByteString String" $ do
-      let f = hush . Witch.tryFrom @LazyByteString.ByteString @String
-      it "works" $ do
-        f (LazyByteString.pack []) `shouldBe` Just ""
-        f (LazyByteString.pack [0x61]) `shouldBe` Just "a"
-        f (LazyByteString.pack [0xff]) `shouldBe` Nothing
-
-    describe "TryFrom LazyByteString String" $ do
-      let f = hush . Witch.tryFrom @(Tagged.Tagged "UTF-8" LazyByteString.ByteString) @String
-      it "works" $ do
-        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just ""
-        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just "a"
-        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
-
     describe "From [Word8] ShortByteString" $ do
       let f = Witch.from @[Word.Word8] @ShortByteString.ShortByteString
       it "works" $ do
@@ -1933,30 +1851,6 @@ spec = describe "Witch" $ do
         f (Text.pack "a") `shouldBe` LazyText.pack "a"
         f (Text.pack "ab") `shouldBe` LazyText.pack "ab"
 
-    describe "From Text ByteString" $ do
-      let f = Witch.from @Text.Text @ByteString.ByteString
-      it "works" $ do
-        f (Text.pack "") `shouldBe` ByteString.pack []
-        f (Text.pack "a") `shouldBe` ByteString.pack [0x61]
-
-    describe "From Text ByteString" $ do
-      let f = Witch.from @Text.Text @(Tagged.Tagged "UTF-8" ByteString.ByteString)
-      it "works" $ do
-        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
-        f (Text.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
-
-    describe "From Text LazyByteString" $ do
-      let f = Witch.from @Text.Text @LazyByteString.ByteString
-      it "works" $ do
-        f (Text.pack "") `shouldBe` LazyByteString.pack []
-        f (Text.pack "a") `shouldBe` LazyByteString.pack [0x61]
-
-    describe "From Text LazyByteString" $ do
-      let f = Witch.from @Text.Text @(Tagged.Tagged "UTF-8" LazyByteString.ByteString)
-      it "works" $ do
-        f (Text.pack "") `shouldBe` Tagged.Tagged (LazyByteString.pack [])
-        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
-
     describe "From LazyText Text" $ do
       let f = Witch.from @LazyText.Text @Text.Text
       it "works" $ do
@@ -1964,30 +1858,6 @@ spec = describe "Witch" $ do
         f (LazyText.pack "a") `shouldBe` Text.pack "a"
         f (LazyText.pack "ab") `shouldBe` Text.pack "ab"
 
-    describe "From LazyText LazyByteString" $ do
-      let f = Witch.from @LazyText.Text @LazyByteString.ByteString
-      it "works" $ do
-        f (LazyText.pack "") `shouldBe` LazyByteString.pack []
-        f (LazyText.pack "a") `shouldBe` LazyByteString.pack [0x61]
-
-    describe "From LazyText LazyByteString" $ do
-      let f = Witch.from @LazyText.Text @(Tagged.Tagged "UTF-8" LazyByteString.ByteString)
-      it "works" $ do
-        f (LazyText.pack "") `shouldBe` Tagged.Tagged (LazyByteString.pack [])
-        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
-
-    describe "From LazyText ByteString" $ do
-      let f = Witch.from @LazyText.Text @ByteString.ByteString
-      it "works" $ do
-        f (LazyText.pack "") `shouldBe` ByteString.pack []
-        f (LazyText.pack "a") `shouldBe` ByteString.pack [0x61]
-
-    describe "From LazyText ByteString" $ do
-      let f = Witch.from @LazyText.Text @(Tagged.Tagged "UTF-8" ByteString.ByteString)
-      it "works" $ do
-        f (LazyText.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
-        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
-
     describe "From String Text" $ do
       let f = Witch.from @String @Text.Text
       it "works" $ do
@@ -2016,30 +1886,6 @@ spec = describe "Witch" $ do
         f (LazyText.pack "a") `shouldBe` "a"
         f (LazyText.pack "ab") `shouldBe` "ab"
 
-    describe "From String ByteString" $ do
-      let f = Witch.from @String @ByteString.ByteString
-      it "works" $ do
-        f "" `shouldBe` ByteString.pack []
-        f "a" `shouldBe` ByteString.pack [0x61]
-
-    describe "From String ByteString" $ do
-      let f = Witch.from @String @(Tagged.Tagged "UTF-8" ByteString.ByteString)
-      it "works" $ do
-        f "" `shouldBe` Tagged.Tagged (ByteString.pack [])
-        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
-
-    describe "From String LazyByteString" $ do
-      let f = Witch.from @String @LazyByteString.ByteString
-      it "works" $ do
-        f "" `shouldBe` LazyByteString.pack []
-        f "a" `shouldBe` LazyByteString.pack [0x61]
-
-    describe "From String LazyByteString" $ do
-      let f = Witch.from @String @(Tagged.Tagged "UTF-8" LazyByteString.ByteString)
-      it "works" $ do
-        f "" `shouldBe` Tagged.Tagged (LazyByteString.pack [])
-        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
-
     describe "From Integer Day" $ do
       let f = Witch.from @Integer @Time.Day
       it "works" $ do
@@ -2165,6 +2011,434 @@ spec = describe "Witch" $ do
       it "works" $ do
         f (Tagged.Tagged False) `shouldBe` Tagged.Tagged False
 
+    describe "From (ISO_8859_1 ByteString) Text" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Text.pack "a"
+
+    describe "From (ISO_8859_1 ByteString) LazyText" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` LazyText.pack "a"
+
+    describe "From (ISO_8859_1 ByteString) String" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` "a"
+
+    describe "From (ISO_8859_1 LazyByteString) LazyText" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` LazyText.pack "a"
+
+    describe "From (ISO_8859_1 LazyByteString) Text" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Text.pack "a"
+
+    describe "From (ISO_8859_1 LazyByteString) String" $ do
+      let f = Witch.from @(Encoding.ISO_8859_1 LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` "a"
+
+    describe "TryFrom Text (ISO_8859_1 ByteString)" $ do
+      let f = hush . Witch.tryFrom @Text.Text @(Encoding.ISO_8859_1 ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Just (Tagged.Tagged $ ByteString.pack [0x61])
+        f (Text.pack "\x100") `shouldBe` Nothing
+
+    describe "TryFrom Text (ISO_8859_1 LazyByteString)" $ do
+      let f = hush . Witch.tryFrom @Text.Text @(Encoding.ISO_8859_1 LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Just (Tagged.Tagged $ LazyByteString.pack [0x61])
+        f (Text.pack "\x100") `shouldBe` Nothing
+
+    describe "TryFrom LazyText (ISO_8859_1 LazyByteString)" $ do
+      let f = hush . Witch.tryFrom @LazyText.Text @(Encoding.ISO_8859_1 LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Just (Tagged.Tagged $ LazyByteString.pack [0x61])
+        f (LazyText.pack "\x100") `shouldBe` Nothing
+
+    describe "TryFrom LazyText (ISO_8859_1 ByteString)" $ do
+      let f = hush . Witch.tryFrom @LazyText.Text @(Encoding.ISO_8859_1 ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Just (Tagged.Tagged $ ByteString.pack [0x61])
+        f (LazyText.pack "\x100") `shouldBe` Nothing
+
+    describe "TryFrom String (ISO_8859_1 ByteString)" $ do
+      let f = hush . Witch.tryFrom @String @(Encoding.ISO_8859_1 ByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Just (Tagged.Tagged $ ByteString.pack [0x61])
+        f "\x100" `shouldBe` Nothing
+
+    describe "TryFrom String (ISO_8859_1 LazyByteString)" $ do
+      let f = hush . Witch.tryFrom @String @(Encoding.ISO_8859_1 LazyByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Just (Tagged.Tagged $ LazyByteString.pack [0x61])
+        f "\x100" `shouldBe` Nothing
+
+    describe "TryFrom (UTF_8 ByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just (Text.pack "a")
+        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
+        f (Tagged.Tagged (ByteString.pack [0x24])) `shouldBe` Just (Text.pack "\x24")
+        f (Tagged.Tagged (ByteString.pack [0xc2, 0xa3])) `shouldBe` Just (Text.pack "\xa3")
+        f (Tagged.Tagged (ByteString.pack [0xe2, 0x82, 0xac])) `shouldBe` Just (Text.pack "\x20ac")
+        f (Tagged.Tagged (ByteString.pack [0xf0, 0x90, 0x8d, 0x88])) `shouldBe` Just (Text.pack "\x10348")
+
+    describe "TryFrom (UTF_8 ByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (LazyText.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just (LazyText.pack "a")
+        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_8 ByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just ""
+        f (Tagged.Tagged (ByteString.pack [0x61])) `shouldBe` Just "a"
+        f (Tagged.Tagged (ByteString.pack [0xff])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_8 LazyByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just (LazyText.pack "")
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just (LazyText.pack "a")
+        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_8 LazyByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just (Text.pack "a")
+        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_8 LazyByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_8 LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [])) `shouldBe` Just ""
+        f (Tagged.Tagged (LazyByteString.pack [0x61])) `shouldBe` Just "a"
+        f (Tagged.Tagged (LazyByteString.pack [0xff])) `shouldBe` Nothing
+
+    describe "From Text (UTF_8 ByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_8 ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
+        f (Text.pack "\x24") `shouldBe` Tagged.Tagged (ByteString.pack [0x24])
+        f (Text.pack "\xa3") `shouldBe` Tagged.Tagged (ByteString.pack [0xc2, 0xa3])
+        f (Text.pack "\x20ac") `shouldBe` Tagged.Tagged (ByteString.pack [0xe2, 0x82, 0xac])
+        f (Text.pack "\x10348") `shouldBe` Tagged.Tagged (ByteString.pack [0xf0, 0x90, 0x8d, 0x88])
+
+    describe "From Text (UTF_8 LazyByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_8 LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (LazyByteString.pack [])
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
+
+    describe "From LazyText (UTF_8 LazyByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_8 LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "") `shouldBe` Tagged.Tagged (LazyByteString.pack [])
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
+
+    describe "From LazyText (UTF_8 ByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_8 ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
+
+    describe "From String (UTF_8 ByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_8 ByteString.ByteString)
+      it "works" $ do
+        f "" `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x61])
+
+    describe "From String (UTF_8 LazyByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_8 LazyByteString.ByteString)
+      it "works" $ do
+        f "" `shouldBe` Tagged.Tagged (LazyByteString.pack [])
+        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61])
+
+    describe "TryFrom (UTF_16LE ByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x24, 0x00])) `shouldBe` Just (Text.pack "\x24")
+        f (Tagged.Tagged (ByteString.pack [0xa3, 0x00])) `shouldBe` Just (Text.pack "\xa3")
+        f (Tagged.Tagged (ByteString.pack [0xac, 0x20])) `shouldBe` Just (Text.pack "\x20ac")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0xd8, 0x48, 0xdf])) `shouldBe` Just (Text.pack "\x10348")
+        f (Tagged.Tagged (ByteString.pack [0x00])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_16LE ByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61, 0x00])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_16LE ByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61, 0x00])) `shouldBe` Just "a"
+
+    describe "TryFrom (UTF_16LE LazyByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_16LE LazyByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00])) `shouldBe` Just (Text.pack "a")
+
+    describe "TryFrom (UTF_16LE LazyByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16LE LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00])) `shouldBe` Just "a"
+
+    describe "From Text (UTF_16LE ByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_16LE ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (Text.pack "\x24") `shouldBe` Tagged.Tagged (ByteString.pack [0x24, 0x00])
+        f (Text.pack "\xa3") `shouldBe` Tagged.Tagged (ByteString.pack [0xa3, 0x00])
+        f (Text.pack "\x20ac") `shouldBe` Tagged.Tagged (ByteString.pack [0xac, 0x20])
+        f (Text.pack "\x10348") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0xd8, 0x48, 0xdf])
+
+    describe "From Text (UTF_16LE LazyByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_16LE LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00])
+
+    describe "From LazyText (UTF_16LE LazyByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_16LE LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00])
+
+    describe "From LazyText (UTF_16LE ByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_16LE ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61, 0x00])
+
+    describe "From String (UTF_16LE ByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_16LE ByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x61, 0x00])
+
+    describe "From String (UTF_16LE LazyByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_16LE LazyByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00])
+
+    describe "TryFrom (UTF_16BE ByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x24])) `shouldBe` Just (Text.pack "\x24")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0xa3])) `shouldBe` Just (Text.pack "\xa3")
+        f (Tagged.Tagged (ByteString.pack [0x20, 0xac])) `shouldBe` Just (Text.pack "\x20ac")
+        f (Tagged.Tagged (ByteString.pack [0xd8, 0x00, 0xdf, 0x48])) `shouldBe` Just (Text.pack "\x10348")
+        f (Tagged.Tagged (ByteString.pack [0x00])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_16BE ByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x61])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_16BE ByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x61])) `shouldBe` Just "a"
+
+    describe "TryFrom (UTF_16BE LazyByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x61])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_16BE LazyByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x61])) `shouldBe` Just (Text.pack "a")
+
+    describe "TryFrom (UTF_16BE LazyByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_16BE LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x61])) `shouldBe` Just "a"
+
+    describe "From Text (UTF_16BE ByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_16BE ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (Text.pack "\x24") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x24])
+        f (Text.pack "\xa3") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0xa3])
+        f (Text.pack "\x20ac") `shouldBe` Tagged.Tagged (ByteString.pack [0x20, 0xac])
+        f (Text.pack "\x10348") `shouldBe` Tagged.Tagged (ByteString.pack [0xd8, 0x00, 0xdf, 0x48])
+
+    describe "From Text (UTF_16BE LazyByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_16BE LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x61])
+
+    describe "From LazyText (UTF_16BE LazyByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_16BE LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x61])
+
+    describe "From LazyText (UTF_16BE ByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_16BE ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x61])
+
+    describe "From String (UTF_16BE ByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_16BE ByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x61])
+
+    describe "From String (UTF_16BE LazyByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_16BE LazyByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x61])
+
+    describe "TryFrom (UTF_32LE ByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x24, 0x00, 0x00, 0x00])) `shouldBe` Just (Text.pack "\x24")
+        f (Tagged.Tagged (ByteString.pack [0xa3, 0x00, 0x00, 0x00])) `shouldBe` Just (Text.pack "\xa3")
+        f (Tagged.Tagged (ByteString.pack [0xac, 0x20, 0x00, 0x00])) `shouldBe` Just (Text.pack "\x20ac")
+        f (Tagged.Tagged (ByteString.pack [0x48, 0x03, 0x01, 0x00])) `shouldBe` Just (Text.pack "\x10348")
+        f (Tagged.Tagged (ByteString.pack [0x00])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_32LE ByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61, 0x00, 0x00, 0x00])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_32LE ByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x61, 0x00, 0x00, 0x00])) `shouldBe` Just "a"
+
+    describe "TryFrom (UTF_32LE LazyByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_32LE LazyByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])) `shouldBe` Just (Text.pack "a")
+
+    describe "TryFrom (UTF_32LE LazyByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32LE LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])) `shouldBe` Just "a"
+
+    describe "From Text (UTF_32LE ByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_32LE ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (Text.pack "\x24") `shouldBe` Tagged.Tagged (ByteString.pack [0x24, 0x00, 0x00, 0x00])
+        f (Text.pack "\xa3") `shouldBe` Tagged.Tagged (ByteString.pack [0xa3, 0x00, 0x00, 0x00])
+        f (Text.pack "\x20ac") `shouldBe` Tagged.Tagged (ByteString.pack [0xac, 0x20, 0x00, 0x00])
+        f (Text.pack "\x10348") `shouldBe` Tagged.Tagged (ByteString.pack [0x48, 0x03, 0x01, 0x00])
+
+    describe "From Text (UTF_32LE LazyByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_32LE LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])
+
+    describe "From LazyText (UTF_32LE LazyByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_32LE LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])
+
+    describe "From LazyText (UTF_32LE ByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_32LE ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x61, 0x00, 0x00, 0x00])
+
+    describe "From String (UTF_32LE ByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_32LE ByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x61, 0x00, 0x00, 0x00])
+
+    describe "From String (UTF_32LE LazyByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_32LE LazyByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x61, 0x00, 0x00, 0x00])
+
+    describe "TryFrom (UTF_32BE ByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE ByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [])) `shouldBe` Just (Text.pack "")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x24])) `shouldBe` Just (Text.pack "\x24")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0xa3])) `shouldBe` Just (Text.pack "\xa3")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x20, 0xac])) `shouldBe` Just (Text.pack "\x20ac")
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x01, 0x03, 0x48])) `shouldBe` Just (Text.pack "\x10348")
+        f (Tagged.Tagged (ByteString.pack [0x00])) `shouldBe` Nothing
+
+    describe "TryFrom (UTF_32BE ByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE ByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x61])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_32BE ByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE ByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x61])) `shouldBe` Just "a"
+
+    describe "TryFrom (UTF_32BE LazyByteString) LazyText" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE LazyByteString.ByteString) @LazyText.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])) `shouldBe` Just (LazyText.pack "a")
+
+    describe "TryFrom (UTF_32BE LazyByteString) Text" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE LazyByteString.ByteString) @Text.Text
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])) `shouldBe` Just (Text.pack "a")
+
+    describe "TryFrom (UTF_32BE LazyByteString) String" $ do
+      let f = hush . Witch.tryFrom @(Encoding.UTF_32BE LazyByteString.ByteString) @String
+      it "works" $ do
+        f (Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])) `shouldBe` Just "a"
+
+    describe "From Text (UTF_32BE ByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_32BE ByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "") `shouldBe` Tagged.Tagged (ByteString.pack [])
+        f (Text.pack "\x24") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x24])
+        f (Text.pack "\xa3") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0xa3])
+        f (Text.pack "\x20ac") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x20, 0xac])
+        f (Text.pack "\x10348") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x01, 0x03, 0x48])
+
+    describe "From Text (UTF_32BE LazyByteString)" $ do
+      let f = Witch.from @Text.Text @(Encoding.UTF_32BE LazyByteString.ByteString)
+      it "works" $ do
+        f (Text.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])
+
+    describe "From LazyText (UTF_32BE LazyByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_32BE LazyByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])
+
+    describe "From LazyText (UTF_32BE ByteString)" $ do
+      let f = Witch.from @LazyText.Text @(Encoding.UTF_32BE ByteString.ByteString)
+      it "works" $ do
+        f (LazyText.pack "a") `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x61])
+
+    describe "From String (UTF_32BE ByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_32BE ByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (ByteString.pack [0x00, 0x00, 0x00, 0x61])
+
+    describe "From String (UTF_32BE LazyByteString)" $ do
+      let f = Witch.from @String @(Encoding.UTF_32BE LazyByteString.ByteString)
+      it "works" $ do
+        f "a" `shouldBe` Tagged.Tagged (LazyByteString.pack [0x00, 0x00, 0x00, 0x61])
+
 newtype Age
   = Age Int.Int8
   deriving (Eq, Show)
@@ -2180,32 +2454,32 @@ type Spec = Writer.Writer (Seq.Seq HUnit.Test) ()
 anyTryFromException :: Selector (Witch.TryFromException s t)
 anyTryFromException = const True
 
-describe :: Stack.HasCallStack => String -> Spec -> Spec
+describe :: (Stack.HasCallStack) => String -> Spec -> Spec
 describe label = testToSpec . HUnit.TestLabel label . specToTest
 
 hush :: Either x a -> Maybe a
 hush = either (const Nothing) Just
 
-it :: Stack.HasCallStack => String -> HUnit.Assertion -> Spec
+it :: (Stack.HasCallStack) => String -> HUnit.Assertion -> Spec
 it label = testToSpec . HUnit.TestLabel label . HUnit.TestCase
 
 shouldBe :: (Stack.HasCallStack, Eq a, Show a) => a -> a -> HUnit.Assertion
 shouldBe = (HUnit.@?=)
 
 shouldSatisfy :: (Stack.HasCallStack, Show a) => a -> (a -> Bool) -> HUnit.Assertion
-shouldSatisfy value predicate = HUnit.assertBool ("predicate failed on: " ++ show value) $ predicate value
+shouldSatisfy value predicate = HUnit.assertBool ("predicate failed on: " <> show value) $ predicate value
 
 shouldThrow :: (Stack.HasCallStack, Exception.Exception e) => IO a -> Selector e -> HUnit.Assertion
 shouldThrow action predicate = do
   result <- Exception.try action
   case result of
     Right _ -> HUnit.assertFailure "did not get expected exception"
-    Left exception -> HUnit.assertBool ("predicate failed on expected exception: " ++ show exception) $ predicate exception
+    Left exception -> HUnit.assertBool ("predicate failed on expected exception: " <> show exception) $ predicate exception
 
-specToTest :: Stack.HasCallStack => Spec -> HUnit.Test
+specToTest :: (Stack.HasCallStack) => Spec -> HUnit.Test
 specToTest = HUnit.TestList . Foldable.toList . Writer.execWriter
 
-testToSpec :: Stack.HasCallStack => HUnit.Test -> Spec
+testToSpec :: (Stack.HasCallStack) => HUnit.Test -> Spec
 testToSpec = Writer.tell . Seq.singleton
 
 unixEpoch :: Time.UTCTime
